import maya.cmds as cmds
import maya.mel as mel
import maya.OpenMayaUI as apiUI
import os
import subprocess
from PySide2 import QtCore, QtUiTools
from PySide2.QtGui import *
from shiboken2 import wrapInstance

import sys

filePath = os.path.dirname(os.path.abspath(__file__))
sysPath = filePath + "\site-packages"
isSysPath = False
for each in sys.path:
    if sysPath in each:
        isSysPath = True
        

if not isSysPath:
    sys.path.append(sysPath)


import certifi
import urllib3
import idna
import charset_normalizer
import requests
import json
import pymel.core as pm

# import importlib
# import IMVUStudioToolkit
# importlib.reload(IMVUStudioToolkit)
# IMVUStudioToolkit.myGui()



headers = {'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/50.0.2661.102 Safari/537.36'}
response = requests.get("https://create.imvu.com/api/studio/maya", headers=headers);
body = json.loads(response.text)
onlineVersion = (body['data']['version'])


leftBracket = "FBXASC091"
rightBracket = "FBXASC093"
period = "FBXASC046"
scaler = 100


def getMayaWindow():
    apiUI.MQtUtil.mainWindow()
    ptr = apiUI.MQtUtil.mainWindow()
    return wrapInstance(int(ptr), QtCore.QObject)

# UI Functions #

class myGui(QtCore.QObject):
    def __init__(self, parent=getMayaWindow()):
        if pm.window("ToolkitWindow", exists= True):
            pm.deleteUI("ToolkitWindow", window= True)
        super(myGui, self).__init__(parent)

        newPath = filePath + "/IMVUStudioToolkit.ui"
        loader = QtUiTools.QUiLoader()
        uifile = QtCore.QFile(newPath)
        uifile.open(QtCore.QFile.ReadOnly)
        self.ui = loader.load(uifile, None)
        self.ui.setWindowFlags(QtCore.Qt.WindowTitleHint)
        self.ui.setObjectName("ToolkitWindow")
        self.ui.setWindowTitle("IMVU Studio Toolkit")
        uifile.close()
        
        if pm.windows.dockControl( 'IMVUStudioToolkit', q = 1, ex = 1 ):
            pm.deleteUI( 'IMVUStudioToolkit')
        window = pm.window(parent= "ToolkitWindow")
        geometry = self.ui.geometry()
        pm.formLayout( parent = window, width= geometry.width(), height= geometry.height() )
        pm.dockControl("IMVUStudioToolkit", area='left', content="ToolkitWindow", allowedArea="left" , width= geometry.width(), height= geometry.height(), floating= False )

        self.ui.TextRoomCustom.installEventFilter(self)

        localVersion = "0.1.3"
        print("Local Version: " + localVersion)
        print("Online Version: " + onlineVersion)

        self.ui.tabWidget.currentChanged.connect(lambda args:DisableButtons(self,"enable"))

        # ROOM TOOL UI #
        self.ui.FloatRoomX.valueChanged.connect(lambda args:ScaleCube(self.ui.FloatRoomX.value(), self.ui.FloatRoomY.value(), self.ui.FloatRoomZ.value()))
        self.ui.FloatRoomY.valueChanged.connect(lambda args:ScaleCube(self.ui.FloatRoomX.value(), self.ui.FloatRoomY.value(), self.ui.FloatRoomZ.value()))
        self.ui.FloatRoomZ.valueChanged.connect(lambda args:ScaleCube(self.ui.FloatRoomX.value(), self.ui.FloatRoomY.value(), self.ui.FloatRoomZ.value()))
        self.ui.ButtonRoomCreate.clicked.connect(lambda args:CreateRoom(self, float(self.ui.FloatRoomX.text()), float(self.ui.FloatRoomY.text()), float(self.ui.FloatRoomZ.text())))
        self.ui.ButtonRoomFurniture.clicked.connect(lambda args:RoomAddFurniture(self, self.ui.FloatRoomX.value(), self.ui.FloatRoomY.value(), self.ui.FloatRoomZ.value()))
        self.ui.RadioRoomFurnNorth.setEnabled(False)
        self.ui.RadioRoomFurnSouth.setEnabled(False)
        self.ui.RadioRoomFurnEast.setEnabled(False)
        self.ui.RadioRoomFurnWest.setEnabled(False)
        self.ui.RadioRoomFurnWall.toggled.connect(lambda args:CheckWall(self, self.ui.RadioRoomFurnWall.isChecked()))
        self.ui.ButtonRoomN.clicked.connect(lambda args:DuplicateFurniture("N", self.ui.FloatRoomOffset.value()))
        self.ui.ButtonRoomS.clicked.connect(lambda args:DuplicateFurniture("S", self.ui.FloatRoomOffset.value()))
        self.ui.ButtonRoomE.clicked.connect(lambda args:DuplicateFurniture("E", self.ui.FloatRoomOffset.value()))
        self.ui.ButtonRoomW.clicked.connect(lambda args:DuplicateFurniture("W", self.ui.FloatRoomOffset.value()))
        self.ui.ButtonRoomIdle.clicked.connect(lambda args:AddSeat("Idle", True, True, "AvatarNodes"))
        self.ui.ButtonRoomStanding.clicked.connect(lambda args:AddSeat("Standing", True, True, "AvatarNodes"))
        self.ui.ButtonRoomSitting.clicked.connect(lambda args:AddSeat("Sitting", True, True, "AvatarNodes"))
        self.ui.ButtonRoomCustom.clicked.connect(lambda args:AddSeat(self.ui.TextRoomCustom.text(), True, True, "AvatarNodes"))
        self.ui.ButtonRoomAddCatcher.clicked.connect(lambda args:AddNode("Add Catch"))
        self.ui.ButtonRoomRemoveCatcher.clicked.connect(lambda args:AddNode("Remove Catch"))
        self.ui.ButtonRoomAddHandle.clicked.connect(lambda args:AddNode("Add Handle"))
        self.ui.ButtonRoomRemoveHandle.clicked.connect(lambda args:AddNode("Remove Handle"))
        self.ui.ButtonRoomCreateST.clicked.connect(lambda args:CreateST("skeleton_Room"))
        self.ui.ButtonRoomLinkST.clicked.connect(LinkST)
        self.ui.ButtonRoomUnlinkST.clicked.connect(UnlinkST)


        # FURNITURE TOOL UI #
        self.ui.ButtonFurnitureCreate.clicked.connect(lambda args:CreateFurniture(self))
        self.ui.ButtonFurnitureIdle.clicked.connect(lambda args:FurnAddSeat("Idle", True, True, "FurnitureNodes"))
        self.ui.ButtonFurnitureStanding.clicked.connect(lambda args:FurnAddSeat("Standing", True, True, "FurnitureNodes"))
        self.ui.ButtonFurnitureSitting.clicked.connect(lambda args:FurnAddSeat("Sitting", True, True, "FurnitureNodes"))
        self.ui.ButtonFurnitureCustom.clicked.connect(lambda args:FurnAddSeat(self.ui.TextFurnitureCustom.text(), True, True, "FurnitureNodes"))
        self.ui.ButtonFurnitureAddCatcher.clicked.connect(lambda args:FurnAddNode("Add Catch"))
        self.ui.ButtonFurnitureRemoveCatcher.clicked.connect(lambda args:FurnAddNode("Remove Catch"))
        self.ui.ButtonFurnitureAddHandle.clicked.connect(lambda args:FurnAddNode("Add Handle"))
        self.ui.ButtonFurnitureRemoveHandle.clicked.connect(lambda args:FurnAddNode("Remove Handle"))
        self.ui.ButtonFurnitureCreateST.clicked.connect(lambda args:CreateST("Attachment_Root"))
        self.ui.ButtonFurnitureLinkST.clicked.connect(LinkST)
        self.ui.ButtonFurnitureUnlinkST.clicked.connect(UnlinkST)
        

        # CLOTHING TOOL UI #
        self.ui.ButtonClothingAppend.clicked.connect(lambda args:ImportClothing(self))
        self.ui.RadioClothingMale.toggled.connect(lambda args:ShowHideAvatar(self.ui.RadioClothingMale.isChecked(), "Male"))
        self.ui.RadioClothingFemale.toggled.connect(lambda args:ShowHideAvatar(self.ui.RadioClothingFemale.isChecked(), "Female"))
        self.ui.RadioClothingHide.toggled.connect(lambda args:ShowHideAvatar(self.ui.RadioClothingHide.isChecked(), "Hide"))
        self.ui.ButtonClothingBind.clicked.connect(BindToAvatar)
        self.ui.ButtonClothingPaint.clicked.connect(PaintMode)
        self.ui.ButtonClothingPose.clicked.connect(PoseMode)
        self.ui.ButtonClothingReset.clicked.connect(ResetPose)
        self.ui.ButtonClothingLimit.clicked.connect(LimitBones)


        # ATTACHMENT TOOL UI #
        self.ui.ButtonAttachmentAppend.clicked.connect(lambda args:ImportAttachment(self))
        self.ui.RadioAttachmentMale.toggled.connect(lambda args:ShowHideAvatar(self.ui.RadioAttachmentMale.isChecked(), "Male"))
        self.ui.RadioAttachmentFemale.toggled.connect(lambda args:ShowHideAvatar(self.ui.RadioAttachmentFemale.isChecked(), "Female"))
        self.ui.RadioAttachmentHide.toggled.connect(lambda args:ShowHideAvatar(self.ui.RadioAttachmentHide.isChecked(), "Hide"))
        self.ui.ButtonAttachmentPick.clicked.connect(lambda args:PickAttachment(self.ui.ComboAttachmentBone))
        self.ui.ButtonAttachmentSetup.clicked.connect(lambda args:SetupAttachment(self.ui.ComboAttachmentBone))
        PopulateDropdown(self.ui.ComboAttachmentBone)


        # ANIMATION TOOL UI #
        # self.ui.ButtonAnimationAppend.clicked.connect(lambda args:ImportAnimation(self))
        # self.ui.IntAnimationLength.valueChanged.connect(lambda args:setAnimLength(self))
        # self.ui.CheckAnimationMakePose
        # self.ui.CheckAnimationExportAll
        # self.ui.CheckAnimationIKLA.toggled.connect(lambda args:ChangeIK_RI(self.ui.CheckAnimationIKLA.isChecked(), "WGT_COG.FK_IK_Arm_L"))
        # self.ui.CheckAnimationIKLA.toggled.connect(lambda args:ChangeIK_RI(self.ui.CheckAnimationIKLA.isChecked(), "WGT_COG.FK_IK_Arm_L"))
        # self.ui.CheckAnimationIKLL.toggled.connect(lambda args:ChangeIK_RI(self.ui.CheckAnimationIKLL.isChecked(), "WGT_COG.FK_IK_Leg_L"))
        # self.ui.CheckAnimationIKRA.toggled.connect(lambda args:ChangeIK_RI(self.ui.CheckAnimationIKRA.isChecked(), "WGT_COG.FK_IK_Arm_R"))
        # self.ui.CheckAnimationIKRL.toggled.connect(lambda args:ChangeIK_RI(self.ui.CheckAnimationIKRL.isChecked(), "WGT_COG.FK_IK_Leg_R"))
        # self.ui.CheckAnimationRILA.toggled.connect(lambda args:ChangeIK_RI(self.ui.CheckAnimationRILA.isChecked(), "WGT_COG.IsoRot_ArmFK_L"))
        # self.ui.CheckAnimationRILL.toggled.connect(lambda args:ChangeIK_RI(self.ui.CheckAnimationRILL.isChecked(), "WGT_COG.IsoRot_LegFK_L"))
        # self.ui.CheckAnimationRIRA.toggled.connect(lambda args:ChangeIK_RI(self.ui.CheckAnimationRIRA.isChecked(), "WGT_COG.IsoRot_ArmFK_R"))
        # self.ui.CheckAnimationRIRL.toggled.connect(lambda args:ChangeIK_RI(self.ui.CheckAnimationRIRL.isChecked(), "WGT_COG.IsoRot_LegFK_R"))
        # self.ui.ButtonAnimationAuto.clicked.connect(lambda args:ToggleAutoKey(self))
        # self.ui.ButtonAnimationKey.clicked.connect(lambda args:AddKey())
        # self.ui.ButtonAnimationZeroBone.clicked.connect(lambda args:ZeroPose())
        # self.ui.ButtonAnimationZeroPose.clicked.connect(lambda args:ZeroBone())


        # EXPORT TOOL UI #
        versionButtonText = "Alpha: " + localVersion

        self.ui.ButtonExport.clicked.connect(Export)
        self.ui.ButtonOpenExport.clicked.connect(OpenExport)
        self.ui.ButtonHelp.clicked.connect(OpenHelp)
        self.ui.ButtonVersion.setText(versionButtonText)
        self.ui.ButtonVersion.clicked.connect(OpenUpdate)
        
        if not localVersion == onlineVersion:
            self.ui.ButtonVersion.setText("UPDATE AVAILABLE!")
            self.ui.ButtonVersion.setStyleSheet('background-color: red ;color: white')
            self.ui.ButtonVersion.setEnabled(True)
        

        # Check Scene #
        for each in cmds.ls():
            if each == "ROOM_EXPORT":
                DisableButtons(self, "room")
            if each == "FURNITURE_EXPORT":
                DisableButtons(self, "furniture")
            if each == "Attachment_Scene":
                DisableButtons(self, "attachment")
            if each == "Clothing_Scene":
                DisableButtons(self, "clothing")
            if each == "Animation_Scene":
                DisableButtons(self, "animation")

        self.ui.show()
        



    def eventFilter(self, obj, event):
        if event.type() == QtCore.QEvent.KeyPress: 
            if event.key() == QtCore.Qt.Key_Return:
                if self.ui.TextRoomCustom.hasFocus():
                    AddSeat(self, self.ui.TextRoomCustom.text(), True, True, "AvatarNodes")
        return super().eventFilter(obj, event)



# General Functions #

def fixLocator(obj):
    x = (cmds.getAttr(obj + ".localPositionX"))
    y = (cmds.getAttr(obj + ".localPositionY"))
    z = (cmds.getAttr(obj + ".localPositionZ"))
    cmds.move(x, y, z, obj, r=1)
    cmds.setAttr(obj + ".localPositionX", 0)
    cmds.setAttr(obj + ".localPositionY", 0)
    cmds.setAttr(obj + ".localPositionZ", 0)
    cmds.xform(obj, cp=1)

def scaleLocator(obj, scaler):
    cmds.setAttr(obj[0] + ".localScaleX", scaler)
    cmds.setAttr(obj[0] + ".localScaleY", scaler)
    cmds.setAttr(obj[0] + ".localScaleZ", scaler)

def ShowHideAvatar(state, type):
    if type == "Male" and state == True:
        for obj in cmds.ls(et="transform"):
            if "Male_Mesh" in obj:
                cmds.showHidden(obj)
            if "Female_Mesh" in obj:
                cmds.hide(obj)
    if type == "Female" and state == True:
        for obj in cmds.ls(et="transform"):
            if "Male_Mesh" in obj:
                cmds.hide(obj)
            if "Female_Mesh" in obj:
                cmds.showHidden(obj)
    if type == "Hide" and state == True:
        for obj in cmds.ls(et="transform"):
            if "Male_Mesh" in obj:
                cmds.hide(obj)
            if "Female_Mesh" in obj:
                cmds.hide(obj)

def CreateST(location):
    global scaler
    cone = cmds.cone(n="SeeThrough01", ax=(0, 0, 1), r=100)
    cmds.move(0, 0, 100)
    cmds.move(0, 0, 0, str(cone[0]) + ".scalePivot", str(cone[0]) + ".rotatePivot")
    cmds.parent(cone[0], location)

def LinkST():
    sel = cmds.ls(sl=1)
    meshes = []
    cone = ""

    for each in sel:
        if (not (cmds.listRelatives(each, typ="nurbsSurface") == None)):
            cone = each
        else:
            meshes.append(each)

    if not cone == "" and not meshes == []:
        for each in meshes:
            cmds.parent(each, cone)

            cmds.select(cone)
            cmds.select(each, add=1)
            cmds.skinCluster(vb=0, vt=1, tst=1, bm=0, mi=1, omi=1, dr=4, nw=1)
            for vtx in range(cmds.polyEvaluate(each, v=1)):
                cmds.skinPercent("skinCluster1", (each + ".vtx[" + str(vtx) + "]"), tv=[(cone, 1)], zri=1)
            
            axis = ['x', 'y', 'z']
            attrs = ['t', 'r', 's']
            for ax in axis:
                for attr in attrs:
                    cmds.setAttr(each+'.'+attr+ax, lock=1)
                    cmds.setAttr(cone+'.'+attr+ax, lock=1)

        cmds.select(d=1)
        cmds.select(cone)

def UnlinkST():
    for each in cmds.ls(sl=1, typ="transform"):
        if (cmds.listRelatives(each, typ="nurbsSurface") == None):
            cmds.DetachSkin()
            axis = ['x', 'y', 'z']
            attrs = ['t', 'r', 's']
            for ax in axis:
                for attr in attrs:
                    cmds.setAttr(cmds.listRelatives(each, p=1)[0]+'.'+attr+ax, lock=0)
            
            cmds.parent(each, cmds.listRelatives(cmds.listRelatives(each, p=1), p=1)[0])
        else:
            cmds.warning("Please select a mesh!")

def PopulateDropdown(dropdownRef):
    isPickBone = False
    if dropdownRef.count() > 0:
        isPickBone = True
    if not isPickBone:
        dropdownRef.addItem("Pick_Bone")
    for obj in cmds.ls(et="transform"):
        if obj == "Attachment_Skeleton":
            for each in reversed(cmds.listRelatives(obj, ad=1, typ="transform")):
                dropdownRef.addItem(each)

def TestCall():
    print("Changed")



# Room Functions #

def ScaleCube(X, Y, Z):
    global scaler
    if not cmds.objExists('Room_Ref'):
        cmds.polyCube(n= "Room_Ref", w= X*scaler, h= Y*scaler, d= Z*scaler)
    if cmds.objExists('Room_Ref'):
        cmds.move(Y*scaler, "Room_Ref.f[1]", y=1, puv=1)
        cmds.move(X*scaler/2, "Room_Ref.f[4]", x=1, puv=1)
        cmds.move(-X*scaler/2, "Room_Ref.f[5]", x=1, puv=1)
        cmds.move(Z*scaler/2, "Room_Ref.f[0]", z=1, puv=1)
        cmds.move(-Z*scaler/2, "Room_Ref.f[2]", z=1, puv=1)

def CreateRoom(self, X, Y, Z):
    global scaler

    createMaterials()
    cmds.modelEditor("modelPanel4", e=1, da="smoothShaded", dtx=1)
    
    # EMPTY GROUPS #
    cmds.group(em=1, n="REFERENCE")
    cmds.group(em=1, n="ROOM_EXPORT")
    
    # SKELETON ROOM #
    skelRoom = cmds.spaceLocator(n= "skeleton_Room")
    cmds.parent("skeleton_Room", "ROOM_EXPORT")
    scaleLocator(skelRoom, scaler)

    # ROOM MESHES #
    if not cmds.objExists('Room_Ref'):
        cmds.polyCube(n= "Room_Ref", w= X*scaler, h= Y*scaler, d= Z*scaler)
        cmds.sets(e=True, forceElement="M_RoomSG")
        cmds.move(0, Y*scaler/2, 0)
        cmds.polyAutoProjection("Room_Ref.f[0:5]", lm=0, pb=0, ibd=1, cm=0, l=2, sc=1, o=1, p=6, ps=0.2, ws=0)
    
    cmds.select("Room_Ref")
    cmds.polyNormal(nm= 0)
    cmds.displayCull( bfc=True )
    cmds.parent("Room_Ref", "REFERENCE")
    dispLayers = cmds.ls(type = 'displayLayer')
    roomExists = False
    for dispLayer in dispLayers:
        if dispLayer == "ReferenceRoom":
            roomExists = True
            layer = dispLayer

    if not roomExists:
        layer = cmds.createDisplayLayer(n= "ReferenceRoom")
        cmds.setAttr("{}.displayType".format(layer), 2)
    else:
        cmds.editDisplayLayerMembers('ReferenceRoom', "Room_Ref")

    cmds.polyCube(n="ScaleCube_Ref", w=3.13*scaler, h=3.13*scaler, d=3.13*scaler)
    cmds.sets(e=True, forceElement="M_CubeSG")
    cmds.move(-8*scaler, 3.13*scaler/2, 1.5*scaler)
    cmds.parent("ScaleCube_Ref", "REFERENCE")
        

    # ROOM NODES #
    camRoot = cmds.spaceLocator(n= "camera_01_01_root")
    cmds.move(0*scaler, 6*scaler, 30*scaler)
    cmds.parent("camera_01_01_root", "skeleton_Room")
    scaleLocator(camRoot, scaler)

    camTarget = cmds.spaceLocator(n= "camera_01_01_target")
    cmds.aimConstraint( 'camera_01_01_target', 'camera_01_01_root', aim=(0, 0, -1))
    cmds.move(0*scaler, 6*scaler, 0*scaler)
    cmds.parent("camera_01_01_target", "skeleton_Room")
    scaleLocator(camTarget, scaler)

    nodeRoom = cmds.spaceLocator(n= "LightingNodes")
    cmds.parent("LightingNodes", "skeleton_Room")
    cmds.ambientLight(n= "Omni01")
    cmds.move(5*scaler, 25*scaler, 0*scaler)
    cmds.parent("Omni01", "LightingNodes")
    scaleLocator(nodeRoom, scaler)
    
    AvatarNodes = cmds.spaceLocator(n= "AvatarNodes")
    cmds.parent("AvatarNodes", "skeleton_Room")
    scaleLocator(AvatarNodes, scaler)
    
    FurnitureNodes = cmds.spaceLocator(n= "FurnitureNodes")
    cmds.parent("FurnitureNodes", "skeleton_Room")
    scaleLocator(AvatarNodes, scaler)

    # POSITION NODES #
    seat1 = cmds.spaceLocator(n= "seat01_Standing")
    cmds.rotate(-90, 0, 0)
    seat2 = cmds.spaceLocator(n= "seat02_Standing")
    cmds.move(3*scaler, 0*scaler, 0*scaler)
    cmds.rotate(-90, 0, 0)
    scaleLocator(seat2, scaler)
    seat3 = cmds.spaceLocator(n= "seat03_Standing")
    cmds.move(6*scaler, 0*scaler, 0*scaler)
    cmds.rotate(-90, 0, 0)
    scaleLocator(seat3, scaler)
    seat4 = cmds.spaceLocator(n= "seat04_Standing")
    cmds.move(9*scaler, 0*scaler, 0*scaler)
    cmds.rotate(-90, 0, 0)
    scaleLocator(seat4, scaler)
    seat5 = cmds.spaceLocator(n= "seat05_Standing")
    cmds.move(12*scaler, 0*scaler, 0*scaler)
    cmds.rotate(-90, 0, 0)
    scaleLocator(seat5, scaler)
    seat6 = cmds.spaceLocator(n= "seat06_Standing")
    cmds.move(15*scaler, 0*scaler, 0*scaler)
    cmds.rotate(-90, 0, 0)
    scaleLocator(seat6, scaler)
    cmds.parent("seat01_Standing", "seat02_Standing", "seat03_Standing", "seat04_Standing", "seat05_Standing", "seat06_Standing", "AvatarNodes")

    cmds.select("ROOM_EXPORT")
    pm.viewFit()
    cmds.setAttr("persp" + "Shape.farClipPlane", 100000)
    cmds.setAttr("persp" + "Shape.nearClipPlane", 10)
    
    DisableButtons(self, "room")

def RoomAddFurniture(self, roomX, roomY, roomZ):
    global scaler

    if self.ui.RadioRoomFurnCeiling.isChecked():
        numCiels = 1
        cielNum = str(numCiels).zfill(2)
        cielList = []

        for each in cmds.ls("FurnitureNodes", dag=1, type=['locator']):
            cielList.append(str(each))

        for o in sorted(cielList):
            if "Ceiling_" + str(numCiels).zfill(2) in o:
                numCiels += 1
                
            seat_str = str(numCiels)
            cielNum = seat_str.zfill(2)

        locator = cmds.spaceLocator(n= ("furniture_Ceiling_" + cielNum))
        cmds.move(0*scaler, roomY*scaler, 0*scaler)
        cmds.rotate(-90, 0, 180)
        scaleLocator(locator, scaler)
        cmds.parent(locator, "FurnitureNodes")
    elif self.ui.RadioRoomFurnFloor.isChecked():
        numFloors = 1
        floorNum = str(numFloors).zfill(2)
        floorList = []

        for each in cmds.ls("FurnitureNodes", dag=1, type=['locator']):
            floorList.append(str(each))

        for o in sorted(floorList):
            if "Floor_" + str(numFloors).zfill(2) in o:
                numFloors += 1
                
            seat_str = str(numFloors)
            floorNum = seat_str.zfill(2)

        locator = cmds.spaceLocator(n= ("furniture_Floor_" + floorNum))
        cmds.move(-5*scaler, 0*scaler, 0*scaler)
        cmds.rotate(-90, 0, 0)
        scaleLocator(locator, scaler)
        cmds.parent(locator, "FurnitureNodes")

    elif self.ui.RadioRoomFurnWall.isChecked():
        numWalls = 1
        wallNum = str(numWalls).zfill(2)
        wallList = []

        for each in cmds.ls("FurnitureNodes", dag=1, type=['locator']):
            wallList.append(str(each))

        for o in sorted(wallList):
            if "Wall_" + str(numWalls).zfill(2) in o:
                numWalls += 1
                
            seat_str = str(numWalls)
            wallNum = seat_str.zfill(2)

        locator = cmds.spaceLocator(n= ("furniture_Wall_" + wallNum))
        
        if self.ui.RadioRoomFurnNorth.isChecked():
            cmds.move(0*scaler, roomY/2*scaler, -roomZ/2*scaler)
        elif self.ui.RadioRoomFurnSouth.isChecked():
            cmds.move(0*scaler, roomY/2*scaler, roomZ/2*scaler)
            cmds.rotate(0, 180, 0)
        elif self.ui.RadioRoomFurnEast.isChecked():
            cmds.move(roomX/2*scaler, roomY/2*scaler, 0*scaler)
            cmds.rotate(0, -90, 0)
        elif self.ui.RadioRoomFurnWest.isChecked():
            cmds.move(-roomX/2*scaler, roomY/2*scaler, 0*scaler)
            cmds.rotate(0, 90, 0)

        scaleLocator(locator, scaler)
        cmds.parent(locator, "FurnitureNodes")

def AddSeat(seatType, ifCall_H, ifCall_CP, collection):
    global scaler
    numSeats = 1
    SeatNum = str(numSeats).zfill(2)

    newList = []

    for each in cmds.ls(collection, dag=1, type=['locator']):
        newList.append(str(each))
    
    for o in sorted(newList):
        if "seat" + str(numSeats).zfill(2) in o:
            numSeats += 1

        seat_str = str(numSeats)
        SeatNum = seat_str.zfill(2)

    seat = cmds.spaceLocator(n= ("seat" + SeatNum + "." + seatType))
    cmds.rotate(-90, 0, 0)
    scaleLocator(seat, scaler)
    cmds.parent(seat, "AvatarNodes")
    
    if ifCall_H:
        handle = cmds.spaceLocator(n= ("Handle" + SeatNum))
        cmds.rotate(-90, 0, 0)
        cmds.move(0, 3*scaler, 0)
        scaleLocator(handle, scaler)
        cmds.parent(handle, seat)
    if ifCall_CP:
        catch = cmds.spaceLocator(n= ("Catcher" + SeatNum + "." + seatType))
        cmds.rotate(-90, 0, 0)
        cmds.move(0, 0, 3*scaler)
        scaleLocator(catch, scaler)
        cmds.parent(catch, seat)
        pitch = cmds.spaceLocator(n= ("Pitcher" + SeatNum + "." + seatType))
        cmds.rotate(-90, 0, 0)
        cmds.move(0, 0, 8*scaler)
        scaleLocator(pitch, scaler)
        cmds.parent(pitch, catch)
    
    cmds.select(seat)
    
def CheckWall(self, enabled):
    self.ui.RadioRoomFurnNorth.setEnabled(enabled)
    self.ui.RadioRoomFurnSouth.setEnabled(enabled)
    self.ui.RadioRoomFurnEast.setEnabled(enabled)
    self.ui.RadioRoomFurnWest.setEnabled(enabled)

def AddNode(nodeType):
    sel = cmds.ls(sl=1)
    if (nodeType == "Remove Handle"):
        for each in sel:
            if "Handle" in each:
                each = cmds.listRelatives(each, p=1)
            for child in cmds.listRelatives(each):
                if "Handle" in child:
                    cmds.delete(child)
                    cmds.select(each)

    if (nodeType == "Remove Catch"):
        for each in sel:
            if "Pitcher" in each:
                each = cmds.listRelatives(each, p=1)
                each = cmds.listRelatives(each, p=1)
            elif "Catcher" in each:
                each = cmds.listRelatives(each, p=1)
            for child in cmds.listRelatives(each):
                if "Catcher" in child:
                    cmds.delete(child)
                    cmds.select(each)

    if (nodeType == "Add Handle"):
        for each in sel:
            isHandle = False
            for child in cmds.listRelatives(each):
                if "Handle" in child:
                    isHandle = True
            if not isHandle:
                prevSel = cmds.ls(sl=1)
                handleNum = each.split("_")[0][4:]
                handle = cmds.spaceLocator(n= ("Handle" + handleNum))

                cmds.rotate(-90, 0, 0)
                cmds.move(cmds.xform(each, piv=1 , q=1 , ws=1)[0], cmds.xform(each, piv=1 , q=1 , ws=1)[1] + (3*scaler), cmds.xform(each, piv=1 , q=1 , ws=1)[2])
                scaleLocator(handle, scaler)
                cmds.parent(handle, each)
                cmds.select(cl=1)
                for sel in prevSel:
                    cmds.select(sel, add=1)

    if (nodeType == "Add Catch"):
        for each in sel:
            isCatch = False
            for child in cmds.listRelatives(each):
                if "Catcher" in child:
                    isCatch = True
            if not isCatch:
                catchNum = each.split("_")[0][4:]
                catchType = each.split("_")[1]

                catch = cmds.spaceLocator(n= ("Catcher" + catchNum + "." + catchType))
                cmds.rotate(-90, 0, 0)
                cmds.move(cmds.xform(each, piv=1 , q=1 , ws=1)[0], cmds.xform(each, piv=1 , q=1 , ws=1)[1], cmds.xform(each, piv=1 , q=1 , ws=1)[2] + (3*scaler))
                scaleLocator(catch, scaler)
                cmds.parent(catch, each)

                pitch = cmds.spaceLocator(n= ("Pitcher" + catchNum + "." + catchType))
                cmds.rotate(-90, 0, 0)
                cmds.move(cmds.xform(each, piv=1 , q=1 , ws=1)[0], cmds.xform(each, piv=1 , q=1 , ws=1)[1], cmds.xform(each, piv=1 , q=1 , ws=1)[2] + (8*scaler))
                scaleLocator(pitch, scaler)
                cmds.parent(pitch, catch)
                cmds.select(each)

def DuplicateFurniture(dir, amt):
    global scaler
    x_amount = 0
    y_amount = 0
    z_amount = 0
    postSel = []
    canDup = True

    if dir == "N":
        y_amount = amt
    if dir == "S":
        y_amount = -amt
    if dir == "E":
        x_amount = amt
    if dir == "W":
        x_amount = -amt


            
    for sel in cmds.ls(sl=1):
        if not "furniture" in sel:
            canDup = False
            cmds.warning("Please select only furniture objects")

    if canDup:
        for sel in cmds.ls(sl=1):
            if "Ceiling" in sel:
                x_amount = -x_amount
            newSel = cmds.duplicate(sel)
            postSel.append(newSel)
            cmds.move(x_amount*scaler, y_amount*scaler, z_amount*scaler, newSel, r=1, os=1)

    for each in postSel:
        cmds.select(each, add=1)

def createMaterials():
    mat = cmds.shadingNode("lambert", asShader=True, name="M_Room")
    cmds.setAttr(mat + ".color", 1, 1, 1)
    matSG = cmds.sets(renderable=True, noSurfaceShader=True, empty=True, name=mat + "SG")
    cmds.connectAttr(mat + ".outColor", matSG + ".surfaceShader", f=True)

    cmds.shadingNode("file", at=1, icm=1)
    cmds.shadingNode("place2dTexture", au=1)
    
    cmds.connectAttr("place2dTexture1.outUV", "file1.uvCoord", f=1)
    cmds.connectAttr("file1.outColor", "M_Room.color", f=1)
    
    scaleRoomPath = (filePath + "/IMVUTextures/TEX_ScaleRoom.png")
    cmds.setAttr("file1.fileTextureName", scaleRoomPath, typ="string")

    
    mat = cmds.shadingNode("lambert", asShader=True, name="M_Cube")
    cmds.setAttr(mat + ".color", 1, 1, 1)
    matSG = cmds.sets(renderable=True, noSurfaceShader=True, empty=True, name=mat + "SG")
    cmds.connectAttr(mat + ".outColor", matSG + ".surfaceShader", f=True)

    cmds.shadingNode("file", at=1, icm=1)
    cmds.shadingNode("place2dTexture", au=1)
    
    cmds.connectAttr("place2dTexture1.outUV", "file2.uvCoord", f=1)
    cmds.connectAttr("file2.outColor", "M_Cube.color", f=1)
    
    scaleCubePath = (filePath + "/IMVUTextures/TEX_ScaleCube.png")
    cmds.setAttr("file2.fileTextureName", scaleCubePath, typ="string")
    



# Furniture Functions #

def CreateFurniture(self):
    global scaler

    createFurnMaterials()
    cmds.modelEditor("modelPanel4", e=1, da="smoothShaded", dtx=1)

    # EMPTY GROUPS #
    cmds.group(em= True, n="REFERENCE")
    cmds.group(em= True, n="FURNITURE_EXPORT")

    # FURNITURE NODES #
    AttachmentRoot = cmds.spaceLocator(n= "Attachment_Root")
    cmds.rotate(-90, 0, 0)
    cmds.parent("Attachment_Root", "FURNITURE_EXPORT")
    scaleLocator(AttachmentRoot, scaler)
    
    # FURNITURE MESHES #
    scaleSeat = cmds.polyCube(n= "ScaleSeat_Ref", w= 3.13*2*scaler, h= 3.13*scaler, d= 3.13*scaler)
    cmds.sets(e=True, forceElement="M_ScaleSeatSG")
    cmds.move(0, 3.13*scaler/2, 0)
    cmds.parent("ScaleSeat_Ref", "REFERENCE")
    cmds.polyAutoProjection("ScaleSeat_Ref.f[0:5]", lm=0, pb=0, ibd=1, cm=0, l=2, sc=1, o=1, p=6, ps=0.2, ws=0)

    cmds.select(scaleSeat)
    pm.viewFit()
    cmds.setAttr("persp" + "Shape.farClipPlane", 100000)
    cmds.setAttr("persp" + "Shape.nearClipPlane", 10)
    DisableButtons(self, "furniture")

def FurnAddSeat(seatType, ifCall_H, ifCall_CP, collection):
    global scaler
    numSeats = 1
    SeatNum = str(numSeats).zfill(2)

    newList = []

    for each in cmds.ls(collection, dag=1, type=['locator']):
        newList.append(str(each))
    
    for o in sorted(newList):
        if "seat" + str(numSeats).zfill(2) in o:
            numSeats += 1

        seat_str = str(numSeats)
        SeatNum = seat_str.zfill(2)

    seat = cmds.spaceLocator(n= ("seat" + SeatNum + "." + seatType))
    cmds.rotate(-90, 0, 0)
    scaleLocator(seat, scaler)
    cmds.parent(seat, "Attachment_Root")
    
    if ifCall_H:
        handle = cmds.spaceLocator(n= ("Handle" + SeatNum))
        cmds.rotate(-90, 0, 0)
        cmds.move(0, 3*scaler, 0)
        scaleLocator(handle, scaler)
        cmds.parent(handle, seat)
    if ifCall_CP:
        catch = cmds.spaceLocator(n= ("Catcher" + SeatNum + "." + seatType))
        cmds.rotate(-90, 0, 0)
        cmds.move(0, 0, 3*scaler)
        scaleLocator(catch, scaler)
        cmds.parent(catch, seat)
        pitch = cmds.spaceLocator(n= ("Pitcher" + SeatNum + "." + seatType))
        cmds.rotate(-90, 0, 0)
        cmds.move(0, 0, 8*scaler)
        scaleLocator(pitch, scaler)
        cmds.parent(pitch, catch)
    
    cmds.select(seat)
    
def FurnAddNode(nodeType):
    sel = cmds.ls(sl=1)
    if (nodeType == "Remove Handle"):
        for each in sel:
            if "Handle" in each:
                each = cmds.listRelatives(each, p=1)
            for child in cmds.listRelatives(each):
                if "Handle" in child:
                    cmds.delete(child)
                    cmds.select(each)

    if (nodeType == "Remove Catch"):
        for each in sel:
            if "Pitcher" in each:
                each = cmds.listRelatives(each, p=1)
                each = cmds.listRelatives(each, p=1)
            elif "Catcher" in each:
                each = cmds.listRelatives(each, p=1)
            for child in cmds.listRelatives(each):
                if "Catcher" in child:
                    cmds.delete(child)
                    cmds.select(each)

    if (nodeType == "Add Handle"):
        for each in sel:
            isHandle = False
            for child in cmds.listRelatives(each):
                if "Handle" in child:
                    isHandle = True
            if not isHandle and "seat" in each:
                prevSel = cmds.ls(sl=1)
                handleNum = each.split("_")[0][4:]
                handle = cmds.spaceLocator(n= ("Handle" + handleNum))

                cmds.rotate(-90, 0, 0)
                cmds.move(cmds.xform(each, piv=1 , q=1 , ws=1)[0], cmds.xform(each, piv=1 , q=1 , ws=1)[1] + (3*scaler), cmds.xform(each, piv=1 , q=1 , ws=1)[2])
                scaleLocator(handle, scaler)
                cmds.parent(handle, each)
                cmds.select(cl=1)
                for sel in prevSel:
                    cmds.select(sel, add=1)

    if (nodeType == "Add Catch"):
        for each in sel:
            isCatch = False
            for child in cmds.listRelatives(each):
                if "Catcher" in child:
                    isCatch = True
            if not isCatch and "seat" in each:
                catchNum = each.split("_")[0][4:]
                catchType = each.split("_")[1]

                catch = cmds.spaceLocator(n= ("Catcher" + catchNum + "." + catchType))
                cmds.rotate(-90, 0, 0)
                cmds.move(cmds.xform(each, piv=1 , q=1 , ws=1)[0], cmds.xform(each, piv=1 , q=1 , ws=1)[1], cmds.xform(each, piv=1 , q=1 , ws=1)[2] + (3*scaler))
                scaleLocator(catch, scaler)
                cmds.parent(catch, each)

                pitch = cmds.spaceLocator(n= ("Pitcher" + catchNum + "." + catchType))
                cmds.rotate(-90, 0, 0)
                cmds.move(cmds.xform(each, piv=1 , q=1 , ws=1)[0], cmds.xform(each, piv=1 , q=1 , ws=1)[1], cmds.xform(each, piv=1 , q=1 , ws=1)[2] + (8*scaler))
                scaleLocator(pitch, scaler)
                cmds.parent(pitch, catch)
                cmds.select(each)

def createFurnMaterials():
    mat = cmds.shadingNode("lambert", asShader=True, name="M_ScaleSeat")
    cmds.setAttr(mat + ".color", 1, 1, 1)
    matSG = cmds.sets(renderable=True, noSurfaceShader=True, empty=True, name=mat + "SG")
    cmds.connectAttr(mat + ".outColor", matSG + ".surfaceShader", f=True)

    cmds.shadingNode("file", at=1, icm=1)
    cmds.shadingNode("place2dTexture", au=1)
    
    cmds.connectAttr("place2dTexture1.outUV", "file1.uvCoord", f=1)
    cmds.connectAttr("file1.outColor", "M_ScaleSeat.color", f=1)
    
    scaleRoomPath = (filePath + "/IMVUTextures/TEX_ScaleSeat.png")
    cmds.setAttr("file1.fileTextureName", scaleRoomPath, typ="string")



# Clothing Functions #

def ImportClothing(self):

    filePath = os.path.dirname(os.path.abspath(__file__))
    clothingPath = filePath + "/IMVUMegaScene/ClothingScene.mb"

    cmds.file(clothingPath, i=1, typ="mayaBinary", iv=1, ra=1, mnc=1, ns=":", op="v=0;", pr=1, ifr=1, itr="override")
    cmds.select("Clothing_Scene")
    pm.viewFit()
    cmds.select(d=1)
    
    DisableButtons(self, "clothing")

def BindToAvatar():
    if (len(cmds.ls(sl=1)) == 0):
        cmds.warning("No Objects Selected")
    elif (len(cmds.ls(sl=1)) > 1):
        cmds.warning("Too Many Objects Selected")
    else:
        clothingOBJ = (cmds.ls(sl=1))

        for obj in cmds.ls(et="transform"):
            if obj == "CLOTHING_EXPORT":
                exportOBJ = obj

            if obj == "Clothing_Skeleton":
                for child in cmds.listRelatives(obj):
                    if child == "Female03MasterRoot":
                        armatureOBJ = child

        isChild = False
        if cmds.listRelatives(exportOBJ, ad=1, typ="transform") == None:
            cmds.parent(clothingOBJ, exportOBJ)
        else:
            for each in cmds.listRelatives(exportOBJ, ad=1, typ="transform"):
                if(clothingOBJ[0] in each):
                    isChild = True
            if (not isChild):
                cmds.parent(clothingOBJ, exportOBJ)

        cmds.select(armatureOBJ)
        cmds.select(clothingOBJ, add=1)
        cmds.SmoothBindSkin()
        cmds.select(clothingOBJ)
        cmds.skinCluster(clothingOBJ, e=1, mi=4)

def PaintMode():
    for obj in cmds.ls(et="transform"):
        if obj == "CLOTHING_EXPORT":
            for child in cmds.listRelatives(obj):
                cmds.select(child)
    cmds.ArtPaintSkinWeightsToolOptions()

def PoseMode():
    for obj in cmds.ls(et="transform"):
        if obj == "Clothing_Skeleton":
            for child in cmds.listRelatives(obj):
                if child == "Female03MasterRoot":
                    cmds.select(child)

def ResetPose():
    for obj in cmds.ls(et="transform"):
        if obj == "Clothing_Skeleton":
            for child in cmds.listRelatives(obj, ad=1):
                cmds.select(child, add=1)
    cmds.joint(e=1, apa=1)
    cmds.select(clear=1)

def LimitBones():
    for obj in cmds.ls(et="transform"):
        if obj == "CLOTHING_EXPORT":
            for child in cmds.listRelatives(obj, ad=1):
                cmds.skinCluster(child, e=1, mi=4)




# Attachment Functions #

def ImportAttachment(self):

    filePath = os.path.dirname(os.path.abspath(__file__))
    clothingPath = filePath + "/IMVUMegaScene/AttachmentScene.mb"

    cmds.file(clothingPath, i=1, typ="mayaBinary", iv=1, ra=1, mnc=1, ns=":", op="v=0;", pr=1, ifr=1, itr="override")
    
    PopulateDropdown(self.ui.ComboAttachmentBone)
    cmds.select("Attachment_Scene")
    pm.viewFit()
    cmds.select(d=1)

    DisableButtons(self, "attachment")

def PickAttachment(dropdownRef):
    attachmentBones = []
    for obj in cmds.ls(et="transform"):
        if obj == "Attachment_Skeleton":
            for each in reversed(cmds.listRelatives(obj, ad=1, typ="transform")):
                attachmentBones.append(each)

    for each in attachmentBones:
        if (cmds.ls(sl=1)[0] == each):
            dropdownRef.setItemText(0, each)

def SetupAttachment(dropdownRef):
    if (dropdownRef.currentText() == "Pick_Bone"):
        dropdownRef.setItemText(0, "Female03MasterRoot")
    if (len(cmds.ls(sl=1)) == 0):
        cmds.warning("No Objects Selected")
    elif (len(cmds.ls(sl=1)) > 1):
        cmds.warning("Too Many Objects Selected")
    else:
        for each in (cmds.ls(sl=1)):
            for child in cmds.listRelatives("ATTACHMENT_EXPORT", ad=1, typ="transform"):
                if(each in child):
                    cmds.parent(each, w=1)
                    
            cmds.matchTransform("AttachmentRoot", dropdownRef.currentText())
            cmds.rotate(180, 0, 90, "AttachmentRoot", r=1, os=1)
            cmds.parent(each, cmds.listRelatives("AttachmentRoot", typ="joint"))

        cmds.rename(cmds.listRelatives("AttachmentRoot", typ="joint"), "AttachmentNode_" + dropdownRef.currentText() + "_")




# Animation Functions #

def ImportAnimation(self):

    filePath = os.path.dirname(os.path.abspath(__file__))
    clothingPath = filePath + "/IMVUMegaScene/AnimationScene.mb"

    cmds.file(clothingPath, i=1, typ="mayaBinary", iv=1, ra=1, mnc=1, ns=":", op="v=0;", pr=1, ifr=1, itr="override")
    DisableButtons(self, "animation")

def setAnimLength(self):
    cmds.playbackOptions(min=0, max=self.ui.IntAnimationLength.value())

def ChangeIK_RI(bool, type):
    cmds.setAttr(type, int(bool == True))

def AddKey():
    cmds.setKeyframe(dd=0, pcs=0, hi="none")
    print(end="Keyframe Added")

def ToggleAutoKey(self):
    cmds.autoKeyframe(state=not cmds.autoKeyframe(q=1, st=1))

    if(cmds.autoKeyframe(q=1, st=1) == True):
        self.ui.ButtonAnimationAuto.setStyleSheet("background-color : red")
    else:
        self.ui.ButtonAnimationAuto.setStyleSheet("background-color : #5b5b5b")

def ZeroPose():
    print("Zero")

def ZeroBone():
    print("Bone")



# Export Functions #

def Export():
    foundExport = False
    if(cmds.file(q=True, sn=True) == ""):
        foundExport = True
        cmds.confirmDialog(m=("File Not Saved"), b="OK")
    else:
        for obj in cmds.ls(et="transform"):
            # ---- ROOM EXPORT ---- #
            if obj == "ROOM_EXPORT": 
                foundExport = True
                cmds.select(obj)
                
                for each in cmds.listRelatives(obj, ad=1):
                    if "_" in each:
                        cmds.rename(each, each.replace("_", period))

                exportLocation = cmds.file(q=True, sn=True)[:-3]
                exportLocation = '"' + exportLocation + '"'
                melCMD = "FBXExport -f " + exportLocation + " -s"
                # set settings
                mel.eval('FBXExportAnimationOnly -v false') 
                mel.eval('FBXExportCameras -v false') 
                mel.eval('FBXExportInAscii -v true') 
                mel.eval('FBXExportScaleFactor 1') 
                mel.eval('FBXExportConvertUnitString cm')
                # export selection
                mel.eval(melCMD)
                
                for each in cmds.listRelatives(obj, ad=1):
                    if period in each:
                        cmds.rename(each, each.replace(period, "_"))
    
                # cmds.warning("Exported Room!")
                cmds.confirmDialog(m=("Room Exported as: " + exportLocation.split("/")[-1][:-1] + ".fbx"), b="OK")

            # ---- FURNITURE EXPORT ---- #
            elif obj == "FURNITURE_EXPORT":
                foundExport = True
                cmds.select(obj)
                for each in cmds.listRelatives(obj, ad=1):
                    if "_" in each:
                        cmds.rename(each, each.replace("_", period))

                exportLocation = cmds.file(q=True, sn=True)[:-3]
                exportLocation = '"' + exportLocation + '"'
                melCMD = "FBXExport -f " + exportLocation + " -s"
                # set settings
                mel.eval('FBXExportAnimationOnly -v false') 
                mel.eval('FBXExportCameras -v false') 
                mel.eval('FBXExportInAscii -v true') 
                mel.eval('FBXExportScaleFactor 1') 
                mel.eval('FBXExportConvertUnitString cm')
                # export selection
                mel.eval(melCMD)

                for each in cmds.listRelatives(obj, ad=1):
                    if period in each:
                        cmds.rename(each, each.replace(period, "_"))

                # cmds.warning("Exported Furniture!")
                cmds.confirmDialog(m=("Furniture Exported as: " + exportLocation.split("/")[-1][:-1] + ".fbx"), b="OK")

            # ---- CLOTHING EXPORT ---- #
            elif obj == "CLOTHING_EXPORT":
                foundExport = True
                cmds.select(obj)
                cmds.select("Clothing_Skeleton", add=1)

                exportLocation = cmds.file(q=True, sn=True)[:-3]
                exportLocation = '"' + exportLocation + '"'
                melCMD = "FBXExport -f " + exportLocation + " -s"
                # set settings
                mel.eval('FBXExportAnimationOnly -v false') 
                mel.eval('FBXExportCameras -v false') 
                mel.eval('FBXExportInAscii -v true') 
                mel.eval('FBXExportScaleFactor 1') 
                mel.eval('FBXExportConvertUnitString cm')
                # export selection
                mel.eval(melCMD)

                # cmds.warning("Exported Clothing!")
                cmds.confirmDialog(m=("Clothing Exported as: " + exportLocation.split("/")[-1][:-1] + ".fbx"), b="OK")

            # ---- ATTATCHMENT EXPORT ---- #
            elif obj == "ATTACHMENT_EXPORT":
                foundExport = True
                boneName = ""
                cmds.select(obj)

                for each in cmds.listRelatives(obj, ad=1):
                    if "AttachmentNode" in each:
                        boneName = each[15:-1]
                        cmds.rename(each, each[:14] + leftBracket + each[15:-1] + rightBracket)
                
                exportLocation = cmds.file(q=True, sn=True)[:-3]
                exportLocation = '"' + exportLocation + '"'
                melCMD = "FBXExport -f " + exportLocation + " -s"
                # set settings
                mel.eval('FBXExportAnimationOnly -v false') 
                mel.eval('FBXExportCameras -v false') 
                mel.eval('FBXExportInAscii -v true') 
                mel.eval('FBXExportScaleFactor 1') 
                mel.eval('FBXExportConvertUnitString cm')
                mel.eval('FBXExportUpAxis z')
                
                # export selection
                mel.eval(melCMD)

                for each in cmds.listRelatives(obj, ad=1):
                    if "AttachmentNode" in each:
                        cmds.rename(each, each[:14] + "_" + boneName + "_")

                # cmds.warning("Exported Attachment!")
                cmds.confirmDialog(m=("Attachment Exported as: " + exportLocation.split("/")[-1][:-1] + ".fbx"), b="OK")
                
            # ---- ANIMATION EXPORT ---- #
            elif obj == "Animation_Scene":
                foundExport = True
                cmds.select("Animation_Skeleton")

                exportLocation = cmds.file(q=True, sn=True)[:-3]
                exportLocation = '"' + exportLocation + '"'
                melCMD = "FBXExport -f " + exportLocation + " -s"
                # set settings
                mel.eval('FBXExportAnimationOnly -v true') 
                mel.eval('FBXExportCameras -v false') 
                mel.eval('FBXExportInAscii -v true') 
                mel.eval('FBXExportScaleFactor 1') 
                
                # export selection
                mel.eval(melCMD)

                # cmds.warning("Exported Attachment!")
                cmds.confirmDialog(m=("Animation Exported as: " + exportLocation.split("/")[-1][:-1] + ".fbx"), b="OK")

    if foundExport == False:
        # cmds.warning("No Export Group Found")
        cmds.confirmDialog(m=("No Export Group Found"), b="OK")

def OpenHelp():
    cmds.showHelp("https://create.imvu.com/articles/studio/using-imvu-studio-toolkit-for-maya/", absolute=True)

def OpenUpdate():
    cmds.showHelp("https://create.imvu.com/articles/studio/toolkit/", absolute=True)

def OpenExport():
    exportLocation = cmds.file(q=True, sn=True)
    exportLocation = exportLocation.replace("/", "\\")
    if(not exportLocation == ""):
        print (exportLocation)
        subprocess.Popen(f'explorer /select,{exportLocation}')
    else:
        cmds.confirmDialog(m=("Please save your file first!"), b="OK")



def DisableButtons(self, buttonType):
    toolType = ""
    for each in cmds.ls():
        if each == "ROOM_EXPORT":
            toolType = "room"
        elif each == "FURNITURE_EXPORT":
            toolType = "furniture"
        elif each == "Attachment_Scene":
            toolType = "attachment"
        elif each == "Clothing_Scene":
            toolType = "clothing"
        # elif each == "Animation_Scene":
        #     toolType = "animation"

    if toolType == "room":
        self.ui.ButtonRoomCreate.setEnabled(False)
        self.ui.ButtonFurnitureCreate.setEnabled(False)
        self.ui.ButtonAttachmentAppend.setEnabled(False)
        self.ui.ButtonClothingAppend.setEnabled(False)
        # self.ui.ButtonAnimationAppend.setEnabled(False)
        buttonType = "room"
    elif toolType == "furniture":
        self.ui.ButtonRoomCreate.setEnabled(False)
        self.ui.ButtonFurnitureCreate.setEnabled(False)
        self.ui.ButtonAttachmentAppend.setEnabled(False)
        self.ui.ButtonClothingAppend.setEnabled(False)
        # self.ui.ButtonAnimationAppend.setEnabled(False)
        buttonType = "furniture"
    elif toolType == "attachment":
        self.ui.ButtonRoomCreate.setEnabled(False)
        self.ui.ButtonFurnitureCreate.setEnabled(False)
        self.ui.ButtonAttachmentAppend.setEnabled(False)
        self.ui.ButtonClothingAppend.setEnabled(False)
        # self.ui.ButtonAnimationAppend.setEnabled(False)
        buttonType = "attachment"
    elif toolType == "clothing":
        self.ui.ButtonRoomCreate.setEnabled(False)
        self.ui.ButtonFurnitureCreate.setEnabled(False)
        self.ui.ButtonAttachmentAppend.setEnabled(False)
        self.ui.ButtonClothingAppend.setEnabled(False)
        # self.ui.ButtonAnimationAppend.setEnabled(False)
        buttonType = "clothing"
    # elif toolType == "animation":
    #     self.ui.ButtonRoomCreate.setEnabled(False)
    #     self.ui.ButtonFurnitureCreate.setEnabled(False)
    #     self.ui.ButtonAttachmentAppend.setEnabled(False)
    #     self.ui.ButtonClothingAppend.setEnabled(False)
    #     self.ui.ButtonAnimationAppend.setEnabled(False)
    #     buttonType = "animation"
    else:
        self.ui.ButtonRoomCreate.setEnabled(True)
        self.ui.ButtonFurnitureCreate.setEnabled(True)
        self.ui.ButtonAttachmentAppend.setEnabled(True)
        self.ui.ButtonClothingAppend.setEnabled(True)
        # self.ui.ButtonAnimationAppend.setEnabled(True)
        buttonType = "enable"

    self.ui.FloatRoomX.setEnabled(                                                      buttonType=="room" or buttonType=="enable")
    self.ui.FloatRoomY.setEnabled(                                                      buttonType=="room" or buttonType=="enable")
    self.ui.FloatRoomZ.setEnabled(                                                      buttonType=="room" or buttonType=="enable")
    # self.ui.ButtonRoomCreate.setEnabled(                                                buttonType=="room" or buttonType=="enable")
    self.ui.ButtonRoomFurniture.setEnabled(                                             buttonType=="room" or buttonType=="enable")
    self.ui.RadioRoomFurnNorth.setEnabled(                                              self.ui.RadioRoomFurnWall.isChecked())
    self.ui.RadioRoomFurnSouth.setEnabled(                                              self.ui.RadioRoomFurnWall.isChecked())
    self.ui.RadioRoomFurnEast.setEnabled(                                               self.ui.RadioRoomFurnWall.isChecked())
    self.ui.RadioRoomFurnWest.setEnabled(                                               self.ui.RadioRoomFurnWall.isChecked())
    self.ui.RadioRoomFurnWall.setEnabled(                                               buttonType=="room" or buttonType=="enable")
    self.ui.RadioRoomFurnFloor.setEnabled(                                              buttonType=="room" or buttonType=="enable")
    self.ui.RadioRoomFurnCeiling.setEnabled(                                            buttonType=="room" or buttonType=="enable")
    self.ui.ButtonRoomN.setEnabled(                                                     buttonType=="room" or buttonType=="enable")
    self.ui.ButtonRoomS.setEnabled(                                                     buttonType=="room" or buttonType=="enable")
    self.ui.ButtonRoomE.setEnabled(                                                     buttonType=="room" or buttonType=="enable")
    self.ui.ButtonRoomW.setEnabled(                                                     buttonType=="room" or buttonType=="enable")
    self.ui.ButtonRoomIdle.setEnabled(                                                  buttonType=="room" or buttonType=="enable")
    self.ui.ButtonRoomStanding.setEnabled(                                              buttonType=="room" or buttonType=="enable")
    self.ui.ButtonRoomSitting.setEnabled(                                               buttonType=="room" or buttonType=="enable")
    self.ui.ButtonRoomCustom.setEnabled(                                                buttonType=="room" or buttonType=="enable")
    self.ui.ButtonRoomAddCatcher.setEnabled(                                            buttonType=="room" or buttonType=="enable")
    self.ui.ButtonRoomRemoveCatcher.setEnabled(                                         buttonType=="room" or buttonType=="enable")
    self.ui.ButtonRoomAddHandle.setEnabled(                                             buttonType=="room" or buttonType=="enable")
    self.ui.ButtonRoomRemoveHandle.setEnabled(                                          buttonType=="room" or buttonType=="enable")
    self.ui.ButtonRoomCreateST.setEnabled(                                              buttonType=="room" or buttonType=="enable")
    self.ui.ButtonRoomLinkST.setEnabled(                                                buttonType=="room" or buttonType=="enable")
    self.ui.ButtonRoomUnlinkST.setEnabled(                                              buttonType=="room" or buttonType=="enable")
    self.ui.TextFurnitureCustom.setEnabled(                                             buttonType=="room" or buttonType=="enable")
    self.ui.FloatRoomOffset.setEnabled(                                                 buttonType=="room" or buttonType=="enable")
    self.ui.TextRoomCustom.setEnabled(                                                  buttonType=="room" or buttonType=="enable")

    # self.ui.ButtonFurnitureCreate.setEnabled(                                           buttonType=="furniture" or buttonType=="enable")
    self.ui.ButtonFurnitureIdle.setEnabled(                                             buttonType=="furniture" or buttonType=="enable")
    self.ui.ButtonFurnitureStanding.setEnabled(                                         buttonType=="furniture" or buttonType=="enable")
    self.ui.ButtonFurnitureSitting.setEnabled(                                          buttonType=="furniture" or buttonType=="enable")
    self.ui.ButtonFurnitureCustom.setEnabled(                                           buttonType=="furniture" or buttonType=="enable")
    self.ui.ButtonFurnitureAddCatcher.setEnabled(                                       buttonType=="furniture" or buttonType=="enable")
    self.ui.ButtonFurnitureRemoveCatcher.setEnabled(                                    buttonType=="furniture" or buttonType=="enable")
    self.ui.ButtonFurnitureAddHandle.setEnabled(                                        buttonType=="furniture" or buttonType=="enable")
    self.ui.ButtonFurnitureRemoveHandle.setEnabled(                                     buttonType=="furniture" or buttonType=="enable")
    self.ui.ButtonFurnitureCreateST.setEnabled(                                         buttonType=="furniture" or buttonType=="enable")
    self.ui.ButtonFurnitureLinkST.setEnabled(                                           buttonType=="furniture" or buttonType=="enable")
    self.ui.ButtonFurnitureUnlinkST.setEnabled(                                         buttonType=="furniture" or buttonType=="enable")
    self.ui.TextFurnitureCustom.setEnabled(                                             buttonType=="furniture" or buttonType=="enable")

    # self.ui.ButtonClothingAppend.setEnabled(                                            buttonType=="clothing" or buttonType=="enable")
    self.ui.RadioClothingMale.setEnabled(                                               buttonType=="clothing" or buttonType=="enable")
    self.ui.RadioClothingFemale.setEnabled(                                             buttonType=="clothing" or buttonType=="enable")
    self.ui.RadioClothingHide.setEnabled(                                               buttonType=="clothing" or buttonType=="enable")
    self.ui.ButtonClothingBind.setEnabled(                                              buttonType=="clothing" or buttonType=="enable")
    self.ui.ButtonClothingPaint.setEnabled(                                             buttonType=="clothing" or buttonType=="enable")
    self.ui.ButtonClothingPose.setEnabled(                                              buttonType=="clothing" or buttonType=="enable")
    self.ui.ButtonClothingReset.setEnabled(                                             buttonType=="clothing" or buttonType=="enable")
    self.ui.ButtonClothingLimit.setEnabled(                                             buttonType=="clothing" or buttonType=="enable")
    
    # self.ui.ButtonAttachmentAppend.setEnabled(                                          buttonType=="attachment" or buttonType=="enable")
    self.ui.RadioAttachmentMale.setEnabled(                                             buttonType=="attachment" or buttonType=="enable")
    self.ui.RadioAttachmentFemale.setEnabled(                                           buttonType=="attachment" or buttonType=="enable")
    self.ui.RadioAttachmentHide.setEnabled(                                             buttonType=="attachment" or buttonType=="enable")
    self.ui.ButtonAttachmentPick.setEnabled(                                            buttonType=="attachment" or buttonType=="enable")
    self.ui.ButtonAttachmentSetup.setEnabled(                                           buttonType=="attachment" or buttonType=="enable")
    self.ui.ComboAttachmentBone.setEnabled(                                             buttonType=="attachment" or buttonType=="enable")
    self.ui.LabelAttachmentBone.setEnabled(                                             buttonType=="attachment" or buttonType=="enable")

    # self.ui.ButtonAnimationAppend.setEnabled(                                          buttonType=="attachment" or buttonType=="enable")
    # self.ui.CheckAnimationMakePose.setEnabled(                                          buttonType=="animation" or buttonType=="enable")
    # self.ui.CheckAnimationExportAll.setEnabled(                                         buttonType=="animation" or buttonType=="enable")
    # self.ui.IntAnimationLength.setEnabled(                                              buttonType=="animation" or buttonType=="enable")
    # self.ui.CheckAnimationIKLA.setEnabled(                                              buttonType=="animation" or buttonType=="enable")
    # self.ui.CheckAnimationIKLL.setEnabled(                                              buttonType=="animation" or buttonType=="enable")
    # self.ui.CheckAnimationIKRA.setEnabled(                                              buttonType=="animation" or buttonType=="enable")
    # self.ui.CheckAnimationIKRL.setEnabled(                                              buttonType=="animation" or buttonType=="enable")
    # self.ui.CheckAnimationRILA.setEnabled(                                              buttonType=="animation" or buttonType=="enable")
    # self.ui.CheckAnimationRILL.setEnabled(                                              buttonType=="animation" or buttonType=="enable")
    # self.ui.CheckAnimationRIRA.setEnabled(                                              buttonType=="animation" or buttonType=="enable")
    # self.ui.CheckAnimationRIRL.setEnabled(                                              buttonType=="animation" or buttonType=="enable")
    # self.ui.ButtonAnimationAuto.setEnabled(                                             buttonType=="animation" or buttonType=="enable")
    # self.ui.ButtonAnimationKey.setEnabled(                                              buttonType=="animation" or buttonType=="enable")
    # self.ui.ButtonAnimationZeroBone.setEnabled(                                         buttonType=="animation" or buttonType=="enable")
    # self.ui.ButtonAnimationZeroPose.setEnabled(                                         buttonType=="animation" or buttonType=="enable")
